# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from .approval_policy_id import ApprovalPolicyId
from .rule import Rule
from .trigger import Trigger

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ApprovalPolicyRequest(pydantic.BaseModel):
    trigger: typing.List[Trigger] = pydantic.Field(
        description="List of triggers that will cause this policy to be evaluated. If no triggers are provided, the policy will be evaluated for all invoices."
    )
    rule: Rule
    upstream_policy_id: ApprovalPolicyId = pydantic.Field(
        alias="upstreamPolicyId",
        description="The policy ID of the previous approval policy in the chain of policies. Use 'root' if no upstreamPolicyId is intended to be set.",
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}
